#!/bin/bash
set -e

usage() {
    echo; echo "Usage: $0 --fasta=/Path/to/infile.fasta --database=/Path/to/NUC_DB --out=/Path/to/output.btab [--evalue=1e-5] [--threads=4]"
    echo "  --fasta     Path to the input nucleotide FASTA file"
    echo "  --out       Path to the output tabular file"
    echo "  --database  Path to the nucleotide BLAST database index base"
    echo "  --evalue    The E value cut-off for the BLAST analysis (Default=1e-5)"
    echo "  --threads   Number of threads to use (Default=1)"
    echo "  -h, --help  Print this help message out"; echo;
    exit 1;
}

## ========================
## Globals
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
PIPELINES="${DIR}"
SCRIPTS="${PIPELINES}/../scripts"
EVALUE="1e-5"
THREADS=1

## ========================
## Process the input parameters
if [ $# -gt 5 ] || [ $# -lt 3 ]
then
    usage
fi

while true
do
    case $1 in
    --help|-h)
	    usage
	    exit;;
    --fasta=?*)
	    FASTA=${1#*=};;
    --fasta|fasta=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --out=?*)
	    OUT=${1#*=};;
    --out|out=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --threads=?*)
	    THREADS=${1#*=};;
    --threads|threads=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --database=?*)
            DATABASE=${1#*=};;
    --database|database=)
            echo "$0: missing argument for '$1' option"
            usage
            exit 1;;
    --evalue=?*)
	    EVALUE=${1#*=};;
    --evalue|evalue=)
	    echo "$0: missing argument for '$1' option"
	    usage
	    exit 1;;
    --)
	    shift
	    break;;
    -?*)
	    echo "$0: invalid option: $1"
	    usage
	    exit 1;;
    *)
	    break
    esac
    shift
done

##===
## Logging
LOG="$( cd "$( dirname "${OUT}" )" && pwd )"
LOGTIME="${LOG}/tblastx.log"

## Run the tBLASTn and output a tabular file
command time -v -o "${LOGTIME}" -a tblastn -query "${FASTA}" \
    -db "${DATABASE}" \
    -out "${OUT}.btab" \
    -evalue "${EVALUE}" \
    -num_threads "${THREADS}" \
    -max_target_seqs 50 \
    -seg no \
    -task 'tblastn-fast' \
    -outfmt 6 > "${LOGTIME}" 2>&1
status=$?

exit $status
