#!/usr/bin/env nextflow

params.fasta = ''
params.working = ''
params.databases = ''
params.threads = 1
params.evalue = 10
params.log = '/dev/null'
params.help = false
params.slurm = false
params.ancestral = false
ancestral_flag = ""


Executor = 'local'

def usage() {
    log.info ''
    log.info 'Usage: nextflow run functional_annotation.nf --fasta /Path/to/infile.fasta --working /Path/to/working_directory --databases /Path/to/databases [--evalue 10] [--threads 4] [--slurm] [--log=/Path/to/run.log]'
    log.info '  --fasta     Path to the input NT FASTA file'
    log.info '  --working   Path to the output working directory'
    log.info "  --databases Path to the SEQSCREEN database directory"
    log.info "  --evalue    E value cut-off (Default=30)"
    log.info "  --threads   Number of threads to use (Default=1)"
    log.info '  --slurm     Submit modules in this workflow to run on a SLURM grid (Default = run locally)'
    log.info "  --ancestral Include all ancestral GO terms"
    log.info "  --log       Where to write the log file to (Default=No log)"
    log.info '  --help      Print this help message out'
    log.info ''
    exit 1
}

if (params.help) {
    usage()
}

if (params.slurm) {
    Executor = 'slurm'
}

if (!params.fasta) {
    log.error "Missing argument for --fasta option"
    usage()
}

if (!params.working) {
    log.error "Missing argument for --working option"
    usage()
}
if (params.ancestral) {
    ancestral_flag = "--ancestral"
}

if (!params.databases) {
    log.error "Missing argument for --databases option"
    usage()
}


fastafile = file(params.fasta)
workingDir = file(params.working)
databasesDir = file(params.databases)
logfile = file(params.log)

BASE=fastafile.getName()
MODULES =  "$workflow.projectDir/../modules"
SCRIPTS = "$workflow.projectDir/../scripts"
THREADS=params.threads
EVALUE=params.evalue
WORKFLOW="functional_annotation"

mummerDir = file("$workingDir/$WORKFLOW/mummer")
megaresDir = file("$workingDir/$WORKFLOW/megares")
fxnDir = file("$workingDir/$WORKFLOW/functional_assignments")

unambigFile = "$workingDir/initialize/ambig_2_unambig/${BASE}.unambig.fasta"
translatedFile = "$workingDir/initialize/six_frame_translation/${BASE}.translated.fasta"

process Create_Working_Directories {
    output:
    stdout create

    """
    mkdir -p $workingDir/$WORKFLOW
    if [ -d $mummerDir ]; then rm -rf $mummerDir; fi;
    if [ -d $megaresDir ]; then rm -rf $megaresDir; fi;
    if [ -d $fxnDir ]; then rm -rf $fxnDir; fi;

    mkdir $mummerDir
    mkdir $megaresDir
    mkdir $fxnDir
    """
}

process Initialize {
    input:
    val create from create

    output:
    stdout init

    executor Executor
    
    script:
    """
    echo -n " # Launching $WORKFLOW workflow ...... " | tee -a $logfile; date '+%H:%M:%S %Y-%m-%d' | tee -a $logfile
    """
}

process mummer {
    input:
    val init from init

    output:
    stdout mummer

    if ( Executor == 'local' ) {
       executor "local"
    }

    else if ( Executor == 'slurm' ) {
       clusterOptions "--ntasks-per-node $THREADS"
       executor "slurm"
    }
    
    """
    ${MODULES}/mummer.sh --fasta=$fastafile \
                         --database=${databasesDir}/rebase/rebase.fna \
                         --out=$mummerDir/${BASE}.mummer_re.txt
    """
}

process megares {
    input:
    val init from init

    output:
    stdout megares

    if ( Executor == 'local' ) {
       executor "local"
    }

    else if ( Executor == 'slurm' ) {
       clusterOptions "--ntasks-per-node $THREADS"
       executor "slurm"
    }
    
    """
    ${MODULES}/blastn.sh --fasta=$fastafile \
                         --database=${databasesDir}/megares/megares_database_v1.01 \
                         --out=$megaresDir/${BASE}.megares \
                         --threads=${THREADS} \
                         --evalue=${EVALUE}
    """
}


process functional_assignments {
    input:
    val mummer from mummer
    val megares from megares

    output:
    stdout functional_assignments

    executor Executor
    
    """
    while [ ! -f $workingDir/taxonomic_identification/blastx/functional_link.ur100.btab ]
    do
      sleep 5
    done

    ${SCRIPTS}/functional_report_generation_sensitive.pl --urbtab=$workingDir/taxonomic_identification/blastx/functional_link.ur100.btab \
                                               --go=${databasesDir}/go/go_network.txt \
                                               --out=${workingDir}/$WORKFLOW/functional_assignments/functional_results.txt \
                                               $ancestral_flag \
                                               --cutoff=3
    echo -n " # $WORKFLOW workflow complete ....... " | tee -a $logfile; date '+%H:%M:%S %Y-%m-%d' | tee -a $logfile
    """
}


create.subscribe { print "$it" }
init.subscribe { print "$it" }
mummer.subscribe { print "$it" }
megares.subscribe { print "$it" }
functional_assignments.subscribe { print "$it" }
